"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.startCloudDeviceInstance = exports.handleInternalRequestError = exports.apiClient = void 0;
const config_1 = require("../config");
const errors_1 = require("../types/errors");
const api_client_1 = require("@cross-lab-project/api-client");
const service_common_1 = require("@crosslab/service-common");
const node_fetch_1 = __importDefault(require("node-fetch"));
exports.apiClient = new api_client_1.APIClient(config_1.config.BASE_URL);
/**
 * TODO: add more information on request/response
 * This function handles errors thrown by an internal api request.
 * @param error The error thrown by the internal api request.
 * @throws {InternalRequestError} Thrown if the error matches an error that can be thrown by an internal api request.
 */
function handleInternalRequestError(error) {
    if (error instanceof api_client_1.FetchError)
        throw new errors_1.InternalRequestError(`An error occurred while trying to fetch the request`, error, 500);
    else if (error instanceof api_client_1.ValidationError)
        throw new errors_1.InternalRequestError(`An error occurred while trying to validate the request/response`, error, 500);
    else if (error instanceof api_client_1.InvalidUrlError)
        throw new errors_1.InternalRequestError(`The provided url is malformed`, error, 400);
    else if (error instanceof api_client_1.UnsuccessfulRequestError)
        throw new errors_1.InternalRequestError(`The request was unsuccessful`, error, error.response.status);
    else
        throw error;
}
exports.handleInternalRequestError = handleInternalRequestError;
/**
 * This function attempts to start the instance of a cloud instantiable device.
 * @param device The cloud instantiable device for which to start the instance.
 * @param deviceUrl The url the instance should connect to as a device.
 * @param token The token used for authentication by the instance.
 * @param experimentUrl The url of the experiment the instance should take part in.
 */
async function startCloudDeviceInstance(device, deviceUrl, token, experimentUrl) {
    try {
        if (!device.instantiateUrl)
            throw new service_common_1.MissingPropertyError('Resolved instantiable cloud device does not have an instantiate url', 500); // NOTE: error code?
        await (0, node_fetch_1.default)(device.instantiateUrl +
            new URLSearchParams([
                ['device_url', deviceUrl],
                ['token', token],
                ['experiment_url', experimentUrl],
            ]), {
            method: 'POST',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', exports.apiClient.accessToken],
            ],
        });
    }
    catch (error) {
        if (error instanceof Error) {
            throw new api_client_1.FetchError(error.message);
        }
        else if (typeof error === 'string') {
            throw new api_client_1.FetchError(error);
        }
        else {
            throw new api_client_1.FetchError('Something went wrong while trying to fetch the request');
        }
    }
}
exports.startCloudDeviceInstance = startCloudDeviceInstance;
//# sourceMappingURL=api.js.map