"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runExperiment = void 0;
const dataSource_1 = require("../../database/dataSource");
const errors_1 = require("../../types/errors");
const api_1 = require("../api");
const url_1 = require("../url");
const book_1 = require("./book");
const service_common_1 = require("@crosslab/service-common");
const service_common_2 = require("@crosslab/service-common");
const setup_1 = require("./setup");
/**
 * This function attempts to run an experiment.
 * @param experimentModel The experiment to be run.
 * @throws {InvalidStateError} Thrown when the status of the experiment is already "finished".
 */
async function runExperiment(experimentModel) {
    const experimentUrl = (0, url_1.experimentUrlFromId)(experimentModel.uuid);
    service_common_2.logger.log('info', 'Attempting to run experiment', { data: { experimentUrl } });
    // make sure experiment is not already finished
    if (experimentModel.status === 'finished') {
        throw new errors_1.InvalidStateError(`Experiment status is already "finished"`, 400);
    }
    // make sure the experiment contains devices
    if (!experimentModel.devices || experimentModel.devices.length === 0) {
        throw new service_common_1.MissingPropertyError(`Experiment does not contain any devices`, 400);
    }
    // book experiment if status is "created"
    if (experimentModel.status === 'created') {
        await (0, book_1.bookExperiment)(experimentModel);
    }
    const resolvedDevices = await Promise.all(experimentModel.devices.map(async (device) => {
        const resolvedDevice = await api_1.apiClient.getDevice(device.url);
        return {
            ...resolvedDevice,
            instanceUrl: device.instance?.url,
            instanceToken: device.instance?.token,
        };
    }));
    await (0, setup_1.setupExperiment)(experimentModel, resolvedDevices);
    // save experiment
    await dataSource_1.repositories.experiment.save(experimentModel);
    service_common_2.logger.log('info', 'Successfully running experiment', { data: { experimentUrl } });
}
exports.runExperiment = runExperiment;
//# sourceMappingURL=run.js.map